/*
Attractions multitouch appliquées à des sprites 
-----------------------------------------------

Même principe que le sketch d'attraction.
En fait la seule modification se passe dans draw()

On obtient le multitouch via 
touches
créé par p5js et qui contient toutes les coordonnées
de tous les doigts en contact avec l'écran

exemple :
touches[0] -> {x:35, y:345, id:1}
ce sont les infos sur le 1er doigt !

S'il n'y a aucun doigt qui touche l'écran,
touches est vide !

Contrairement à mousePressed() / touchstart(),
et mouseX / touchX
cette fois-ci ça ne marche QUE en tactile;
rien à la souris.
*/

// une variable Array pour stocker les  exemplaires de fantômes
var ghosts = [];

function setup(){ 

	createCanvas(windowWidth, windowHeight);

	// on ajoute 30 fantômes
	for (var i = 0; i < 30; i++) {
		addGhost(random(0, width), random(0, height));
	};
}
function draw(){
	background('#29adff'); // nettoyer l'affichage à chaque cycle

	// on boucle sur tous les doigts enregistrés
    for(var j = 0; j < touches.length; j++){
		
		// et on applique l'attraction vers ce doigt à tous les Sprites p5play
		for (var i = 0; i < ghosts.length; i++) {
			// touches[j] c'est le doigt en cours
			// dont on a x et y livrés par p5js
			attire(ghosts[i], touches[j].x, touches[j].y, 200, 1, 20);

			// c'est tout pour le multitouch !
		}        
    }

    // on empêche les fantômes de sortir de l'écran
    for (var i = 0; i < ghosts.length; i++) {

		var g = ghosts[i];

		// ce coup-ci il sortent par un côté et reviennent de l'autre
		if(g.position.x<0){
			g.position.x += width;
		} else if(g.position.x>width){
			g.velocity.x -= width;
		}
		if(g.position.y<0){
			g.position.y += height;
		} else if(g.position.y>height){
			g.position.y -= height;
		}
	}

	// !!! imortant !!! rafraichir l'affichage de tous les Sprites p5play
	drawSprites();

	// aficher le mode d'emploi !
	text("// Attraction de sprites p5play !", 30, 30);
	text("// NE FONCTIONNE QU'AVEC UN ECRAN TACTILE !", 30, 50);
}

function addGhost(x, y){ // ajoute un fantôme de plus, là où on veut

	// on crée un Sprite, vide
	var ghost = createSprite(x, y, 30, 30); // x, y, L, H
  	ghost.addAnimation("vole", "img/fighter1.png", "img/fighter3.png", "img/fighter2.png", "img/fighter3.png");
	ghost.changeAnimation("vole");

	// on peut changer sa taille au hasard
	ghost.scale = random(0.5, 1);

	// si on veut qu'il se déplace, c'est prévu !
	ghost.velocity.x = random(-1, 1);
	ghost.velocity.y = random(-1, 1);

	// on ajoute le fantôme à la "collection" de fantômes
	ghosts.push(ghost);

	// la fonction addGhost a fabriqué un fantôme,
	// et elle nous y donne accès si on veut
	return ghost;
}

/* -------------------
ATTRACTION !

l'idée c'est de créer une zone autour d'un point (le doigt/souris ou un autre point)
dans cette zone les fantômes sont attirés,
hors de cette zone les fantômes vivent leur vie comme avant.

une fonction d'attraction ré-utilisable !

les paramètres à donner :
- une sprite p5play - obtenu via createSprite(), ici ça se passe dans addGhost()
- une coordonnée X du point à partir duquel on attire
- une coordonnée y du point à partir duquel on attire
- le rayon de la zone de répulsion
- le facteur d'atténuation (permet d'adoucir ou durcir la répulsion)
	1 c'est assez réactif, 0.5 très réactif, 4 plus doux par exemple
- la distance minimum : trop près ça ne sert à rien d'attirer, ça crée des bugs d'affichage...
	je n'ai pas trouvé mieux pour l'instant comme solution !
	entre 10 et 30 c'est pas mal.
*/
function attire(sprite, pointX, pointY, ray, attenuation, distmin){

	// d'abord on regarde la distance entre le sprite et le point x/y
	var distance = dist(pointX, pointY, sprite.position.x, sprite.position.y);
	// http://p5js.org/reference/#/p5/dist

	// si le sprite est assez près :
	if(distance < ray && distance > distmin){

		// affichage d'une ligne pour se rendre compte... à virer plus tard !
		line(pointX, pointY, sprite.position.x, sprite.position.y);

		// on détermine le "vecteur" entre la souris et le sprite http://p5js.org/reference/#/p5.Vector
		var vect = createVector(pointX-sprite.position.x, pointY-sprite.position.y);
		// remarque : c'est l'opération inverse de la répulsion !

		/*
		plus on est près plus on doit être attiré,
		ça tombe mal, plus on est prêt plus le vecteur est court...

		il va falloir "inverser" la longueur du vecteur :
		- petite distance devient grand vecteur
		- grande distance devient petit vecteur
		*/

		var longueur = ray/distance;

		// changer la longueur du vecteur et garder sa direction
		vect.setMag(longueur); 

		// on n'a plus qu'à appliquer cette force au sprite !
		sprite.position.x += vect.x/attenuation;
		sprite.position.y += vect.y/attenuation; 			

		/* 
		l'atténuation met un peu de douceur...
		et permet de moduler la réactivité de la répulsion
		*/

		/*
		on aurait aussi pu toucher à la vitesse du sprite via
		g.velocity.x
		g.velocity.y
		mais là il peut reprendre sa vitesse initiale, qu'on n'a pas modifié,
		une fois sorti du "champ" de la souris/doigt !
		*/

	}
}